/****************************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,                          */
/*                       All Rights Reserved.                                           */
/*                                                                                      */
/* Name:    mo_compare_input_data                                                       */
/*                                                                                      */
/* Purpose: Compare two MO input data definitions                                       */
/*                                                                                      */
/* Input:   original_input_data_id     - "From" input_data                              */
/*          input_data_id              - "To" input_data                                */
/*                                                                                      */
/* Output:  &_rc = return code                                                          */
/*                                                                                      */
/* HISTORY:                                                                             */
/* 08Mar05 S0237007 Creation (dwh)                                                      */
/* 27Jun05 S0306987 Create dummy format if no data to remap (dwh)                       */
/*                                                                                      */
/* 4.4:                                                                                 */
/* 19Dec05 S0336504 Add customer_attribute as by variable when comparing                */
/*                  customer_attribute_values (dwh)                                     */
/* 18Apr06 S0354306 Add channel_units to channel table compare (dwh)                    */
/* 11May06 S0358467 Set flag whenever a customer_attribute_value disappears (dwh)       */
/* 11May06 S0358743 Upgrade return code processing (dwh)                                */
/* 19May06 S0360168 QUIT after proc format in case it fails (dwh)                       */
/* 12Jun06 S0363384 Remove upcase from value comparisions (dwh)                         */
/*                                                                                      */
/* 5.3:                                                                                 */
/* 11Dec08 S0483854 Add support for a new special value for the time period IDs to be   */
/*         interpreted as "use the extreme/outermost time period values from the Time   */ 
/*         Periods table to define the beginning and end of the time scope over which   */
/*         to apply a contact policy" (KEC)                                             */
/* 30Jun09 S0542237 If there is a cell code field associated with the input data, then  */
/*         in the code sections that process communication values and communication     */
/*         numeric attribute values, respectively, perform an additional SQL step which */
/*         joins the cell code value to each row of the working table PLUS include the  */
/*         cell code value in the "by" values passed to macro compare_input_data. (KEC) */
/****************************************************************************************/

%macro mo_compare_input_data(original_input_data_id        =
                            ,input_data_id                 =
                            ,_rc                           =
                            );

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %global &_rc;
   %let &_rc = 0;

   %put %mo_note Starting comparison of input data definitions.;
   %mo_constants

   %*--------------------------------;
   %*  Useful macros                 ;
   %*--------------------------------;
   %macro compare_input_data(lib   = mometa
                            ,table =
                            ,key   =
                            ,by    =
                            ,other =
                            ,_rc   =
                            );

      %if %bquote(&_rc) eq %str() %then %do;
         %local rc;
         %let _rc = rc;
      %end;
      %else %global &_rc;
      %let &_rc = 0;

      %mo_no_notes(off)

      proc contents data=&lib..&table out=cont_out noprint;
      run;

      %local char_vars;
      proc sql noprint;
         select upcase(name) into :char_vars separated by '!'
         from work.cont_out
         where (type eq 2);
      quit;

      %if %length(&key) eq 32 %then %let shortkey = %substr(&key,1,31);
      %else %let shortkey = &key;

      data work.&table.1;
         set &lib..&table (rename=(&key=&shortkey.1));
         where (input_data_id eq &original_input_data_id);
         %do i=1 %to %mo_word_count(&by);
            %let var=%scan(%upcase(&by),&i);
            %if (%sysfunc(indexw(&char_vars,&var,!))) %then %do;
               &var = trim(&var);
            %end;
         %end;
         keep &shortkey.1 &by &other;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred creating work.&table.1 (syserr=&syserr).;
      %end;

      /* When working with the Time Periods table, if time periods exist in  */
      /* the original (base) input data, add a row for the special value     */
      /* used to represent earliest/latest time period.                      */
      %if (%upcase(&table) eq TIME_PERIODS) %then %do; 
         %if (%mo_obs_in(work.&table.1)) %then %do;
            data work.&table.1;
               set work.&table.1 end=last;
               if (last = 1) then do;
                  output;

                  &shortkey.1 = &_CONTACT_POLICY_TP_TABLE_RANGE_;
                  &by = " ";
                  output;
               end;
               else do;
                  output;
               end;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred appending earliest/latest (&_CONTACT_POLICY_TP_TABLE_RANGE_) time period value to work.&table.1 (syserr=&syserr).;
            %end;
         %end;
      %end;

      proc sort data=work.&table.1 out=work.base;
         by &by;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
      %end;

      data work.&table.2;
         set &lib..&table (rename=(&key=&shortkey.2));
         where (input_data_id eq &input_data_id);
         %do i=1 %to %mo_word_count(&by);
            %let var=%scan(%upcase(&by),&i);
            %if (%sysfunc(indexw(&char_vars,&var,!))) %then %do;
               &var = trim(&var);
            %end;
         %end;
         keep &shortkey.2 &by &other;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred creating work.&table.2 (syserr=&syserr).;
      %end;

      /* When working with the Time Periods table, if time periods exist in  */
      /* the new (comparison) input data, add a row for the special value    */
      /* used to represent earliest/latest time period.                      */
      %if (%upcase(&table) eq TIME_PERIODS) %then %do; 
         %if (%mo_obs_in(work.&table.2)) %then %do;
            data work.&table.2;
               set work.&table.2 end=last;
               if (last = 1) then do;
                  output;

                  &shortkey.2 = &_CONTACT_POLICY_TP_TABLE_RANGE_;
                  &by = " ";
                  output;
               end;
               else do;
                  output;
               end;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred appending earliest/latest (&_CONTACT_POLICY_TP_TABLE_RANGE_) time period value to work.&table.2 (syserr=&syserr).;
            %end;
         %end;
      %end;

      proc sort data=work.&table.2 out=work.comp;
         by &by;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
      %end;

      data work.&table work.&table._old work.&table._new;
         merge work.base (in=a) work.comp (in=b);
         by &by;
         if (a) then output work.&table;
         if (a and b) then output work.&table._old;
         if (b and not a) then output work.&table._new;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred creating work.&table (syserr=&syserr).;
      %end;

      %if (%mo_obs_in(work.&table)) %then %do;
         data work.map (keep=label start fmtname type);
            set work.&table;
            label=&shortkey.2;
            start=&shortkey.1;
            fmtname="&key";
            type='I';
         run;
      %end;
      %else %do;
         data work.map (keep=label start fmtname type);
            length fmtname $32.;
            length label start 8.;
            length type $1.;
            label=.;
            start=.;
            fmtname="&key";
            type='I';
         run;
      %end;

      %mo_no_notes(on)
      proc format cntlin=work.map;
      quit;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred creating &key format (syserr=&syserr).;
      %end;

   %mend compare_input_data;


   %*--------------------------------;
   %*  Compare tables                ;
   %*--------------------------------;

   %compare_input_data(lib   = mometa
                      ,table = channels
                      ,key   = channel_id
                      ,by    = channel channel_type
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  channels (cprc=&cprc).;
   %end;


   %compare_input_data(lib   = mometa
                      ,table = campaigns
                      ,key   = campaign_id
                      ,by    = campaign_code
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  campaigns (cprc=&cprc).;
   %end;


   /* Determine if this is MOMA input data and therefore has the cell_cd      */
   /* column present in the input data.  If so, the cell_cd value must be     */
   /* included by the comparisons / format creation for the communications    */
   /* table and for the comm_numeric_attributes_d table.                      */
   /* Note: It is sufficient to get this from just one of the input data      */
   /* entries specified.                                                      */
   %local cell_cd_column_status;
   proc sql noprint;
      select cell_cd_column_status into :cell_cd_column_status
      from mometa.input_data
      where (input_data_id = &input_data_id);
   quit;
   %if ((&sqlrc gt 4) or (&sqlobs eq 0)) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while reading metadata (sqlrc=&sqlrc, sqlobs=&sqlobs).;
      %goto ERROR_EXIT;
   %end;
   %let cell_cd_column_status = %trim(%left(&cell_cd_column_status));


   %if ((&cell_cd_column_status eq &_CELL_CD_UNIQUE_) or
        (&cell_cd_column_status eq &_CELL_CD_UNIQUE_UNNEEDED_)) %then %do;
      proc sql noprint;
         create table work.temp_comm as
         select a.*, b.campaign_code
         from mometa.communications as a left join mometa.campaigns as b
         on a.campaign_id eq b.campaign_id;

         create table work.communications as
         select a.*, d.comm_attribute_value as cell_code
         from work.temp_comm as a,
              mometa.comm_character_attribute_values as b,
              mometa.comm_character_attributes as c,
              mometa.comm_character_distinct_values as d
         where (a.campaign_id = b.campaign_id) and
               (a.communication_id = b.communication_id) and
               (b.comm_char_attribute_id = c.comm_char_attribute_id) and
               (lowcase(c.comm_attribute_name) = "cell_cd") and
               (b.comm_attribute_distinct_value_id = d.comm_attribute_distinct_value_id);
      quit;
   %end;
   %else %do;
      proc sql noprint;
         create table work.communications as
         select a.*, b.campaign_code
         from mometa.communications as a left join mometa.campaigns as b
         on a.campaign_id eq b.campaign_id;
      quit;
   %end;

   %compare_input_data(lib   = work
                      ,table = communications
                      ,key   = communication_id
                      ,by    = campaign_code communication_code
                   %if ((&cell_cd_column_status eq &_CELL_CD_UNIQUE_) or
                        (&cell_cd_column_status eq &_CELL_CD_UNIQUE_UNNEEDED_)) %then %do;
                               cell_code
                   %end;
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  communications (cprc=&cprc).;
   %end;

   
   data work.comm_numeric_attributes;
      set mometa.comm_numeric_attributes;
      comm_attribute_name = upcase(comm_attribute_name);
   run;
   
   %compare_input_data(lib   = work
                      ,table = comm_numeric_attributes
                      ,key   = comm_attribute_id
                      ,by    = comm_attribute_name
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  comm_numeric_attributes (cprc=&cprc).;
   %end;

   %if ((&cell_cd_column_status eq &_CELL_CD_UNIQUE_) or
        (&cell_cd_column_status eq &_CELL_CD_UNIQUE_UNNEEDED_)) %then %do;
      proc sql noprint;
         create table work.temp_comm_numeric_attr_d as
         select a.*, b.campaign_code, c.communication_code, upcase(d.comm_attribute_name) as comm_attribute_name
         from mometa.comm_numeric_attribute_defaults as a, mometa.campaigns as b,
              mometa.communications as c, mometa.comm_numeric_attributes as d
         where a.campaign_id eq b.campaign_id and a.communication_id eq c.communication_id
           and a.comm_attribute_id eq d.comm_attribute_id;

         create table work.comm_numeric_attribute_d as
         select a.*, d.comm_attribute_value as cell_code
         from work.temp_comm_numeric_attr_d as a,
              mometa.comm_character_attribute_values as b,
              mometa.comm_character_attributes as c,
              mometa.comm_character_distinct_values as d
         where (a.campaign_id = b.campaign_id) and
               (a.communication_id = b.communication_id) and
               (b.comm_char_attribute_id = c.comm_char_attribute_id) and
               (lowcase(c.comm_attribute_name) = "cell_cd") and
               (b.comm_attribute_distinct_value_id = d.comm_attribute_distinct_value_id);
      quit;
   %end;
   %else %do;
      proc sql noprint;
         create table work.comm_numeric_attribute_d as
         select a.*, b.campaign_code, c.communication_code, upcase(d.comm_attribute_name) as comm_attribute_name
         from mometa.comm_numeric_attribute_defaults as a, mometa.campaigns as b,
              mometa.communications as c, mometa.comm_numeric_attributes as d
         where a.campaign_id eq b.campaign_id and a.communication_id eq c.communication_id
           and a.comm_attribute_id eq d.comm_attribute_id;
      quit;
   %end;

   %compare_input_data(lib   = work
                      ,table = comm_numeric_attribute_d
                      ,key   = comm_attribute_value_id
                      ,by    = campaign_code communication_code 
                   %if ((&cell_cd_column_status eq &_CELL_CD_UNIQUE_) or
                        (&cell_cd_column_status eq &_CELL_CD_UNIQUE_UNNEEDED_)) %then %do;
                               cell_code
                   %end;
                               comm_attribute_name comm_attribute_value
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  comm_numeric_attribute_defaults (cprc=&cprc).;
   %end;


   data work.comm_character_attributes;
      set mometa.comm_character_attributes;
      comm_attribute_name = upcase(comm_attribute_name);
   run;

   %compare_input_data(lib   = work
                      ,table = comm_character_attributes
                      ,key   = comm_char_attribute_id
                      ,by    = comm_attribute_name
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  comm_character_attributes (cprc=&cprc).;
   %end;


   proc sql noprint;
      create table work.comm_character_distinct_v as
      select upcase(a.comm_attribute_name) as comm_attribute_name, b.*
      from mometa.comm_character_attributes as a,
           mometa.comm_character_distinct_values as b
      where a.comm_char_attribute_id eq b.comm_char_attribute_id;
   quit;

   %compare_input_data(lib   = work
                      ,table = comm_character_distinct_v
                      ,key   = comm_attribute_distinct_value_id
                      ,by    = comm_attribute_name comm_attribute_value
                      ,other = comm_char_attribute_id
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  comm_character_distinct_values (cprc=&cprc).;
   %end;


   data work.customer_attributes;
      set mometa.customer_attributes;
      customer_attribute = upcase(customer_attribute);
   run;

   %compare_input_data(lib   = work
                      ,table = customer_attributes
                      ,key   = customer_attribute_id
                      ,by    = customer_attribute attribute_type
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  customer_attributes (cprc=&cprc).;
   %end;


   proc sql noprint;
      create table work.customer_attribute_values as
      select upcase(a.customer_attribute) as customer_attribute, b.*
      from mometa.customer_attributes as a,
           mometa.customer_attribute_values as b
      where a.customer_attribute_id eq b.customer_attribute_id;
   quit;

   %compare_input_data(lib   = work
                      ,table = customer_attribute_values
                      ,key   = customer_attribute_value_id
                      ,by    = customer_attribute customer_attribute_value
                      ,other = customer_attribute_id
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  customer_attribute_values (cprc=&cprc).;
   %end;


   %compare_input_data(lib   = mometa
                      ,table = time_periods
                      ,key   = time_period_id
                      ,by    = time_period
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  time_periods (cprc=&cprc).;
   %end;


   data work.customer_offer_attributes;
      set mometa.customer_offer_attributes;
      customer_offer_attribute = upcase(customer_offer_attribute);
   run;

   %compare_input_data(lib   = work
                      ,table = customer_offer_attributes
                      ,key   = customer_offer_attribute_id
                      ,by    = customer_offer_attribute
                      ,_rc   = cprc
                      );
   %if (&cprc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred comparing input data records for this table:  customer_offer_attributes (cprc=&cprc).;
   %end;



   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %* Successful completion;
   %put;
   %if (&&&_rc eq 0) %then %do;
      %put %mo_note Successfully compared input data = &original_input_data_id to input data = &input_data_id..;
      %goto EXIT;
   %end;


   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered during comparison of input data = &original_input_data_id to input data = &input_data_id (rc=&&&_rc).;


   %EXIT:


%mend mo_compare_input_data;
