/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_delete_metadata_tables                                       */
/*                                                                          */
/* Purpose: Delete the specified table(s) from the file system directoy and */
/*          from the SAS library in the OMR.                                */
/*                                                                          */
/*          Alternatively, delete the entire contents of the file system    */
/*          directory and the SAS library.                                  */
/*                                                                          */
/* Input:   library_metadata_id - Metadata identifier of input data library */
/*                                                                          */
/*          all_tables - Delete specified tables in table_list (0) or all   */
/*          tables (1)                                                      */
/*                                                                          */
/*          table_list - List of tables to be deleted (if all_tables is 0)  */
/*                                                                          */
/* Output:  &_rc = return code                                              */
/*                                                                          */
/* HISTORY:                                                                 */
/* 19Jun08 S0483854 MOMA Support - Creation (KEC)                           */
/*                                                                          */
/* F1MO54, F0MO541                                                          */
/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */
/*         of SAS Proc Contents.                                            */
/*                                                                          */
/****************************************************************************/

%macro mo_delete_metadata_tables(library_metadata_id =
                                ,all_tables          = 0
                                ,table_list          =
                                ,_rc                 =
                                );
                           
   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %do;
      %global &_rc;
   %end;
   %let &_rc = 0;

   %mo_constants


   /* Validate parameters.                                                  */
   %if (%bquote(&library_metadata_id) eq %str()) %then %do;
      %put %mo_error No library_metadata_id was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&all_tables) eq %str()) %then %do;
      %put %mo_error Ambiguous request - Parameter all_tables is blank.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if ((&all_tables eq 0) and (%bquote(&table_list) eq %str())) %then %do;
      %put %mo_error No table_list was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;

   /* Parse library metadata ID for use by PROC METALIB (9.2).              */
   %local repositoryId libraryId;
   %let repositoryId = %scan(&library_metadata_id, 1, .);
   %let libraryId = %scan(&library_metadata_id, 2, .);
   %if ((&repositoryId eq ) or (&libraryId eq )) %then %do;
      %put %mo_error Incorrectly formatted metadata library ID.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;

   /* Obtain file system path name corresponding to the specified library   */
   /* metadata ID.                                                          */
   %mo_no_notes(off)      
   %mo_get_metadata_objects(objtype   = Directory
                           ,search    = *[UsedByPackages/SASLibrary[@Id='&library_metadata_id']]
                           ,attribute = DirectoryName
                           ,length    = 4096
                           ,out       = work.mo_path
                           ,_rc       = omrrc
                           );
   %if (&omrrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error retrieving directory metadata from OMR (omrrc=&omrrc).;
      %goto ERROR_EXIT;
   %end;

   /* Dump the directory name into a path local macro variable.              */
   %local path;
   %let path =;
   proc sql noprint;
      select DirectoryName into :path
      from work.mo_path;
   quit;
   %if ((&sqlrc gt 4) or (&sqlobs eq 0)) %then %do;
      %put %mo_error SQL error retrieving directory metadata.;
      %let &_rc = 107;
      %goto ERROR_EXIT;
   %end;
   %let path=%trim(%left(&path));

   /* Assign a library reference to the obtained path.                       */
   %mo_no_notes(on)
   %put;
   libname _tmplib_ "&path";
   %if (&syslibrc ne 0) %then %do;
      %put %mo_error Error assigning _TMPLIB_ libref (syslibrc=&syslibrc);
      %let &_rc = 109;
      %goto ERROR_EXIT;
   %end;
   %mo_no_notes(off)


   /* If "all tables" was specified, create a list of the SAS tables in the  */
   /* directory.                                                             */
   %if (&all_tables ne 0) %then %do;     
      proc contents data=_tmplib_._all_ out=work.tmpLibAll(keep=memname) noprint nodetails;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered using proc contents to get the list of tables in _TMPLIB_ (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      proc sql noprint;
         select distinct memname into :table_list separated by ' '
         from work.tmpLibAll;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %put %mo_error SQL error encountered retrieving list of tables from WORK.TMPLIBALL (sqlrc=&sqlrc).;
         %let &_rc = 107;
         %goto ERROR_EXIT;
      %end;

      %let table_list = %lowcase(&table_list);
   %end;
   %put %mo_note The SAS tables located in directory %upcase(&path) to be deleted are %upcase(&table_list).;


   /* We could do all tables in the list at one time, but chose to deal with */
   /* each one individually so that any error for a given table can be       */
   /* separately recorded.                                                   */
   %local wcval;
   %let wcval = %mo_word_count(&table_list);
   %local i;
   %local table_name;

   /* First, physically delete the SAS table file from the directory.        */
   %if (%bquote(&table_list) ne %str()) %then %do;
      %do i = 1 %to &wcval;
         %let table_name = %scan(&table_list, &i);

         /* Delete the specified dataset. Note: Chose not to use             */
         /* mo_delete_file_item due to processing overhead.                  */
         %if (%sysfunc(exist(_tmplib_.&table_name))) %then %do; /* This check needed when user passes in a list of tables */
            %mo_no_notes(off)            
            proc datasets library=_tmplib_ nolist;
               delete &table_name;
            quit;
            %mo_no_notes(on)            
            %if (&syserr gt 4) %then %do;
               %let &_rc = 115;
               %put %mo_error Error encountered while deleting the %upcase(&table_name) table in directory %upcase(&path) (syserr=&syserr).;
               %goto ERROR_EXIT;
            %end;
            %else %do;
               %put;
               %put %mo_note Deleted table %upcase(&table_name) in directory %upcase(&path) (syserr=&syserr).;
            %end;
         %end;
         %else %do;
            %put;
            %put %mo_note The %upcase(&table_name) table does not exist in file system directory %upcase(&path).;
         %end;
      %end;
   %end;

   /* Then, de-register the table definitions from the metadata.             */
   %do i = 1 %to &wcval;
      %let table_name = %scan(&table_list, &i);
      %mo_no_notes(off)            
      proc metalib;
         omr (libid="&libraryId" repid="&repositoryId");
         select (&table_name);
         update_rule = (delete);
      run;
      %mo_no_notes(on)            
      %if (&syserr gt 4) %then %do;
         %let &_rc = 113;
         %put %mo_error Error encountered while de-registering the %upcase(&table_name) table definition metadata (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      %else %do;
          %put;
          %put %mo_note Deleted table definition %upcase(&table_name) from the metadata repository.;
      %end;
   %end;


   /* If this was an "all tables" request, delete the directory from the     */
   /* file system and the SASLibrary from the OMR.                           */
   %if (&all_tables = 1) %then %do;
      %if (%sysfunc(fileexist(&path))) %then %do;
         %mo_delete_file_item(item = &path
                             ,_rc = dirrc);
         %if (&dirrc ne 0) %then %do;
            %let &_rc = 115;
            %put %mo_error I/O error encountered while deleting directory %upcase(&path) (dirrc=&dirrc).;
            %goto ERROR_EXIT;
         %end;
         %else %do;
            %put;
            %put %mo_note Deleted directory %upcase(&path).;
         %end;
      %end;
      %else %do;
         %put;
         %put %mo_note The directory %upcase(&path) does not exist.;
      %end;

      %let omrrc = 0;
      data _null_;
         length treeURI $ 1024;
         length anyURI $ 1024;

         treeURI = "0";
         anyURI = "0";

         /* Get the SAS Folder in which this SASLibrary resides.             */
         rcGAL = metadata_getnasn("OMSOBJ:SASLibrary/&library_metadata_id", "Trees", "1", treeURI);
         if ((rcGAL lt 0) and (rcGAL ne -4)) then do;
            %mo_handle_metadata_operation_rc(oper=GTASN,objTp=SASLibrary,objRf=&library_metadata_id,asn=Trees,asnTp=Tree,repos=&repositoryId,rcVar=rcGAL);
         end;

         rcDL = metadata_delobj("omsobj:SASLibrary/&library_metadata_id");
         %mo_handle_metadata_operation_rc(oper=DEL,objTp=SASLibrary,objRf=&library_metadata_id,repos=&repositoryId,rcVar=rcDL);

         /* If the SASLibrary was the only object in the SAS Folder, i.e.,   */
         /* there are no other objects associated with the Folder, then      */
         /* delete the Folder as well.                                       */
         rcGAT = metadata_getnasn(treeURI, "Members", "1", anyURI);
         if ((rcGAT lt 0) and (rcGAT ne -4)) then do;
            %mo_handle_metadata_operation_rc(oper=GTASN,objTp=Tree,objRf=TBD,asn=Members,asnTp=TBD,repos=&repositoryID,rcVar=rcGAT);
         end;

         rcDT = 0;
         if (rcGAT le 0) then do;
            rcDT = metadata_delobj(treeURI);
            %mo_handle_metadata_operation_rc(oper=DEL,objTp=Tree,objRf=,repos=&repositoryId,rcVar=rcDT);
         end;

         if ((rcDL ne 0) and (rcDT ne 0)) then do;
            call symput("omrrc", put(100, 8.));
         end;
      run;
      %if (&omrrc ne 0) %then %do;
         %let &_rc = 113;
         %put %mo_error Error encountered while attempting to delete SASLibrary (&library_metadata_id) (omrrc=&omrrc).;
         %goto ERROR_EXIT;
      %end;
      %else %do;
         %put;
         %put %mo_note Deleted SASLibrary %upcase(&library_metadata_id).;
      %end;
   %end;


   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %put;
   %local put_string;
   %if (&all_tables = 0) %then %do;
      %let put_string = the following tables - %upcase(&table_list) -;
   %end;
   %else %do;
      %let put_string = all tables; 
   %end;

   %if (&&&_rc eq 0) %then %do;
      %put %mo_note Successfully deleted &put_string from directory %upcase(&path), metadata ID=&library_metadata_id.;
   %end;
   %else %do;
      %put %mo_warning Encountered errors while deleting &put_string from directory %upcase(&path), metadata ID=&library_metadata_id (rc=&&&_rc).;
   %end;
   %goto EXIT;

   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered while deleting tables (rc=&&&_rc).;

   %EXIT:
   %* Clean up;
   libname _tmplib_ clear;

%mend mo_delete_metadata_tables;
