/****************************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,                          */
/*                       All Rights Reserved.                                           */
/*                                                                                      */
/* Name:    mo_validate_imported_solution                                               */
/*                                                                                      */
/* Purpose: Validate imported solution dataset                                          */
/*                                                                                      */
/* Input    lib                 - libname                                               */
/*          ds                  - dataset name                                          */
/*          cust_id             - customer id column                                    */
/*          metadata_id         - SASLibrary FQID                                       */
/*          communication_table - for this input data                                   */
/*                                                                                      */
/* Output:  &_rc = return code                                                          */
/*                                                                                      */
/* Return Codes:                                                                        */
/*          0     completed validation with no warnings nor errors detected             */
/*          4     completed validation with warnings but no errors detected             */
/*          8     completed validation with errors detected                             */
/*                                                                                      */
/*                                                                                      */
/* HISTORY:                                                                             */
/* 09Apr04 S0244828 Creation (dwh)                                                      */
/* 02Sep04 S0266653 Send import data validation errors to client (dwh)                  */
/* 13Sep04 S0267783 Make column check case insensitive (dwh)                            */
/*                                                                                      */
/* 4.3:                                                                                 */
/* 11Oct04 S0256473 Convert GUI messages for I18N (dwh)                                 */
/* 07Dec04 S0268539 Handle leading blanks (dwh)                                         */
/* 24Mar05 S0273454 User OMR fqid (dwh)                                                 */
/* 27May05 S0302470 Disallow embedded or leading spaces in camp_code, comm_code (dwh)   */
/* 08Jun05 S0303659 Limit the number of error messages that get printed (dwh)           */
/*                                                                                      */
/* 4.4:                                                                                 */
/* 02Jun06 S0361315 Check for blanks in case reading from Oracle (dwh)                  */
/*                                                                                      */
/* 5.3:                                                                                 */
/* 17Jul08 S0483854 MOMA support - Add support for cell_cd column when importing a      */
/*         solution into project that has cell_cd in the Communications and Customer    */
/*         Control input data tables. (KEC)                                             */
/* 15Aug08 S0483854 MOMA support - Add validation checks for multi-part key customer ID */
/*         column name. (KEC)                                                           */
/****************************************************************************************/

%macro mo_validate_imported_solution(lib                 =
                                    ,ds                  =
                                    ,cust_id             =
                                    ,cell_cd_status      =
                                    ,communication_table =
                                    ,_rc                 =
                                    );

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %global &_rc;
   %let &_rc = 0;

   %mo_no_notes(off)

   %local max_messages;
   %let max_messages=100;


   %* Set up validation message file;
   data _null_;
      length path $4096.;
      path=pathname('WORK');
      call symput('workpath',strip(path));
   run;

   %global message_file;
   %let message_file=&workpath.&separator.mo_messages.txt;

   %if (%sysfunc(fileexist("&message_file"))) %then %do;
      %mo_delete_file_item(item=&message_file,_rc=drc)
      %if (&drc ne 0) %then %do;
         %let &_rc = 200;
         %put %mo_warning Error deleting mo_messages.txt.;
      %end;
   %end;

   filename momsg "&message_file";
   %if (&sysfilrc ne 0 and &sysfilrc ne 1) %then %do;
      %put %mo_error Error assigning MOMSG filref (sysfilrc=&sysfilrc);
      %let &_rc = 125;
      %goto ERROR_EXIT;
   %end;

   %local setfile;
   %let setfile= file momsg lrecl=512 mod;



   %*--------------------------------;
   %* Check table/column attributes  ;
   %*--------------------------------;
   %mo_create_tables(lib=work, table=INPUT, _rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred creating INPUT tables in WORK library.;
      %goto ERROR_EXIT;
   %end;

   %mo_no_notes(off)
   proc contents data=&lib..&ds out=contents_new (keep=name type length) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of &lib..&ds (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.contents_new;
      set work.contents_new;
      name=upcase(name);
   run;

   proc sort data=contents_new;
      by name;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.contents_new (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc contents data=work.communication out=contents_old (keep=name type length
                 where=(lowcase(name) in ('campaign_cd' 'communication_cd' 
                 %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
                                          'cell_cd'
                 %end;
                                         ))) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of work.communication (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.contents_old;
      set work.contents_old;
      name=upcase(name);
   run;

   proc sort data=contents_old;
      by name;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.contents_old (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data compare;
      length from $8.;
      merge work.contents_new (in=a rename=(type=type_new length=length_new))
            work.contents_old (in=b rename=(type=type_old length=length_old));
      by name;
      name=upcase(name);
      if (a and b) then from = 'both';
      else if (a) then from = 'new';
      else from='old';
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.compare (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   /* Parse customer_id_column_name into sub-components in case it is a */
   /* multi-part key.                                                   */
   %local numCustIdCols custIdColNum;
   %local custIdColNameListQuote custIdColNameListComma;
   %let custIdColNameListQuote =;
   %let custIdColNameListComma =;
   %let numCustIdCols = %mo_word_count(&cust_id, %str(*));
   %do custIdColNum = 1 %to &numCustIdCols;
      %local custIdCol&custIdColNum;
      %let custIdCol&custIdColNum = %scan(&cust_id, &custIdColNum, *);
      %let custIdColNameListQuote = %nrbquote(&custIdColNameListQuote)%str(",")&&custIdCol&custIdColNum;
      %let custIdColNameListComma = &custIdColNameListComma, &&custIdCol&custIdColNum;
   %end;
   %let custIdColNameListQuote = %substr(%nrbquote(&custIdColNameListQuote), 4);
   %let custIdColNameListComma = %substr(%nrbquote(&custIdColNameListComma), 3);

   data _null_;
      set compare;
      &setfile;

      if ((from eq 'new') and (lowcase(name) not in ("%lowcase(%nrbquote(&custIdColNameListQuote))"))) then do;

         putlog "%mo_note Additional column detected in %upcase(&lib..&ds) table: " name;
         putlog ' ';
      end;
      if ((from eq 'old') and 
          ((lowcase(name) eq "campaign_cd") or 
           (lowcase(name) eq "communication_cd")
        %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
           or (lowcase(name) eq "cell_cd") 
        %end;
          )) then do;
         putlog "%mo_error Missing column detected in %upcase(&lib..&ds) table: " name;
         put    "input.data.validation.missing.column.error.message.fmt.txt,2,%upcase(&lib..&ds)," name;
         putlog ' ';
         call symput("&_rc",'8');
      end;
      if (from eq 'both') then do;
         if (type_new ne type_old) then do;
            putlog "%mo_error Column is defined with the incorrect type in %upcase(&lib..&ds) table: " name;
            put    "input.data.validation.invalid.column.type.error.message.fmt.txt,2,%upcase(&lib..&ds)," name;
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length_new lt length_old) then do;
            if (type_new eq 1) then do;
               putlog "%mo_warning Numeric column is defined with length shorter than 8. in %upcase(&lib..&ds) table: " name;
               putlog ' ';
            end;
            else if (type_new eq 2) then do;
               putlog "%mo_warning Character column is defined with length shorter than expected in %upcase(&lib..&ds) table: " name;
               putlog ' ';
            end;
         end;
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.compare (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %do custIdColNum = 1 %to &numCustIdCols;
      proc sql noprint;
         select *
         from work.contents_new
         where (upcase(name) eq "%upcase(&&custIdCol&custIdColNum)");
      quit;
      %if (&sqlobs eq 0) %then %do;
         %let &_rc = 8;
         %put %mo_error Customer ID column = &&custIdCol&custIdColNum not found in %upcase(&lib..&ds).;
         data _null_;
            &setfile;
            put "input.data.validation.missing.customer_id.column.error.message.fmt.txt,2,&&custIdCol&custIdColNum,%upcase(&lib..&ds)";
         run;
      %end;
   %end;
   %if (&&&_rc eq 8) %then %goto BOTTOM;


   %*--------------------------------;
   %* Create useful strings          ;
   %*--------------------------------;
   %local campaign_cds;
   %local communication_cds;
   proc sql noprint;
      select distinct(quote(trim(campaign_cd))) into :campaign_cds separated by ','
         from &communication_table;
      select distinct(quote(trim(communication_cd))) into :communication_cds separated by ','
         from &communication_table;
   %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
      select distinct(quote(trim(cell_cd))) into :cell_cds separated by ','
         from &communication_table;
   %end;
   quit;

   %if (%bquote(&campaign_cds) eq %str()) %then %let campaign_cds='';
   %if (%bquote(&communication_cds) eq %str()) %then %let communication_cds='';
   %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
      %if (%bquote(&cell_cds) eq %str()) %then %let cell_cds='';
   %end;


   %*--------------------------------;
   %* Check imported solution data   ;
   %*--------------------------------;
   data work.&ds;
      set &lib..&ds;
      retain error_count 0;
      &setfile;

      if (error_count gt &max_messages) then do;
         putlog "%mo_note Maximum number of import data validation errors reached.";
         stop;
      end;

      if (&custIdCol1 eq ' ') 
         %do custIdColNum=2 %to &numCustIdCols; /* Blank values are allowed   */
         and (&&custIdCol&custIdColNum eq ' ')  /* in some, but not all, key  */
         %end;                                  /* columns in a multi-part    */
         then do;                               /* customer ID.               */
         putlog "%mo_error Blank value detected for %upcase(&custIdColNameListComma) in observation " _n_ "of %upcase(&lib..&ds) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,%upcase(&custIdColNameListComma)," _N_ +(-1)",%upcase(&lib..&ds)";
         putlog ' ';
         call symput("&_rc",'8');
         error_count+1;
      end;

      if (campaign_cd eq ' ') then do;
         putlog "%mo_error Blank value detected for CAMPAIGN_CD in observation " _n_ "of %upcase(&lib..&ds) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&lib..&ds)";
         putlog ' ';
         call symput("&_rc",'8');
         error_count+1;
      end;
      else do;
         if (substr(campaign_cd,1,1) eq ' ') then do;
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&lib..&ds) table must not have leading blanks.";
            put    "input.data.validation.leading.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
         if (anyspace(substr(campaign_cd,1,length(campaign_cd))) gt 0) then do;
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&lib..&ds) table must not have embedded spaces.";
            put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
         if (campaign_cd not in (&campaign_cds)) then do;
            putlog "%mo_error Invalid CAMPAIGN_CD detected in observation " _n_ "of %upcase(&lib..&ds) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
      end;

      if (communication_cd eq ' ') then do;
         putlog "%mo_error Blank value detected for COMMUNICATION_CD in observation " _n_ "of %upcase(&lib..&ds) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&lib..&ds)";
         putlog ' ';
         call symput("&_rc",'8');
         error_count+1;
      end;
      else do;
         if (substr(communication_cd,1,1) eq ' ') then do;
            putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&lib..&ds) table must not have leading blanks.";
            put    "input.data.validation.leading.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
         if (anyspace(substr(communication_cd,1,length(communication_cd))) gt 0) then do;
            putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&lib..&ds) table must not have embedded spaces.";
            put    "input.data.validation.embedded.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
         if (communication_cd not in (&communication_cds)) then do;
            putlog "%mo_error Invalid COMMUNICATION_CD detected in observation " _n_ "of %upcase(&lib..&ds) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
      end;

      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
         if (cell_cd eq ' ') then do;
            putlog "%mo_error Blank value detected for CELL_CD in observation " _n_ "of %upcase(&lib..&ds) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&lib..&ds)";
            putlog ' ';
            call symput("&_rc",'8');
            error_count+1;
         end;
         else do;
            if (substr(cell_cd,1,1) eq ' ') then do;
               putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&lib..&ds) table must not have leading blanks.";
               put    "input.data.validation.leading.space.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&lib..&ds)";
               putlog ' ';
               call symput("&_rc",'8');
               error_count+1;
            end;
            if (anyspace(substr(cell_cd,1,length(cell_cd))) gt 0) then do;
               putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&lib..&ds) table must not have embedded spaces.";
               put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&lib..&ds)";
               putlog ' ';
               call symput("&_rc",'8');
               error_count+1;
            end;
            if (cell_cd not in (&cell_cds)) then do;
               putlog "%mo_error Invalid CELL_CD detected in observation " _n_ "of %upcase(&lib..&ds) table.";
               put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&lib..&ds)";
               putlog ' ';
               call symput("&_rc",'8');
               error_count+1;
            end;
         end;
      %end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered validating &lib..&ds (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sort data=&communication_table out=work.communication_table nodupkey;
      by campaign_cd communication_cd
      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
         cell_cd
      %end;
      ;
      where ((campaign_cd ne '') and (communication_cd ne '')
      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
             and (cell_cd ne '')
      %end;
            );
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.communication_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sort data=&lib..&ds out=work.&ds nodupkey;
      by campaign_cd communication_cd
      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
         cell_cd
      %end;
      ;
      where ((campaign_cd ne '') and (communication_cd ne '')
      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
             and (cell_cd ne '')
      %end;
            );
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.&ds (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.compare;
      length from $8.;
      merge work.communication_table (in=b) work.&ds (in=a);
      by campaign_cd communication_cd
      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
         cell_cd
      %end;
      ;
      if (a and b) then from = 'both';
      else if (a) then from = 'cntl';
      else from='comm';
      keep campaign_cd communication_cd from
      %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
           cell_cd
      %end;
      ;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.compare (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data _null_;
      set work.compare;
      retain error_count 0;
      &setfile;

      if (error_count gt &max_messages) then do;
         %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
            putlog "%mo_note Maximum number of CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD validation errors reached.";
         %end;
         %else %do;
            putlog "%mo_note Maximum number of CAMPAIGN_CD/COMMUNICATION_CD validation errors reached.";
         %end;
         stop;
      end;

      if (from eq 'cntl') then do;
         putlog ' ';
         %if (&cell_cd_status ne &_CELL_CD_NOT_PRESENT_) %then %do;
            putlog "%mo_error Invalid CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combination detected in %upcase(&lib..&ds) table where CAMPAIGN_CD = " campaign_cd ", COMMUNICATION_CD = " communication_cd  ", and CELL_CD = " cell_cd ".";
            put    "input.data.validation.invalid.combination.cell.error.message.fmt.txt,10,CAMPAIGN_CD,COMMUNICATION_CD,CELL_CD,%upcase(&lib..&ds),CAMPAIGN_CD," campaign_cd +(-1)",COMMUNICATION_CD," communication_cd +(-1)",CELL_CD," cell_cd;
         %end;
         %else %do;
            putlog "%mo_error Invalid CAMPAIGN_CD/COMMUNICATION_CD combination detected in %upcase(&lib..&ds) table where CAMPAIGN_CD = " campaign_cd " and COMMUNICATION_CD = " communication_cd ".";
            put    "input.data.validation.invalid.combination.error.message.fmt.txt,7,CAMPAIGN_CD,COMMUNICATION_CD,%upcase(&lib..&ds),CAMPAIGN_CD," campaign_cd +(-1)",COMMUNICATION_CD," communication_cd;
         %end;
         putlog ' ';
         call symput("&_rc",'8');
         error_count+1;
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.compare (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   %mo_no_notes(on)

   %BOTTOM:

   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %put;
   %if (&&&_rc eq 0) %then
      %put %mo_note Imported solution data validation completed with no errors and no warnings.;
   %else %if (&&&_rc eq 4 or &&&_rc eq 200) %then
      %put %mo_warning Imported solution data validation completed with no errors but with warnings (rc=&&&_rc).;
   %else
      %put %mo_error Imported solution data validation detected errors (rc=&&&_rc).;
   %put;
   %goto EXIT;

   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered while validating inported solution (rc=&&&_rc).;


   %EXIT:

%mend mo_validate_imported_solution;
